import { __decorate, __metadata } from 'tslib';
import { Component, ViewEncapsulation, Renderer2, ElementRef, Input, ContentChildren, QueryList, ViewChild, NgModule } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { fromEvent } from 'rxjs';
import { debounceTime } from 'rxjs/operators';

let HsSlideComponent = class HsSlideComponent {
    constructor(_renderer, _element) {
        this._renderer = _renderer;
        this._element = _element;
    }
    SetWidth(width) {
        this._renderer.setStyle(this._element.nativeElement, "width", width.toString() + "px");
    }
};
HsSlideComponent = __decorate([
    Component({
        selector: 'hs-slide, [hs-slide]',
        template: "<ng-content></ng-content>",
        encapsulation: ViewEncapsulation.None,
        styles: ["[hs-slide],hs-slide{display:flex;flex-direction:column;justify-content:center;align-items:stretch;overflow:hidden}"]
    }),
    __metadata("design:paramtypes", [Renderer2, ElementRef])
], HsSlideComponent);

let SlidesContainerComponent = class SlidesContainerComponent {
    constructor(_renderer, _element) {
        this._renderer = _renderer;
        this._element = _element;
        this._containerOffsetLeft = 0;
    }
    ngAfterContentInit() {
    }
    SetVisibleSlides(visibleSlides) {
        this.VisibleSlides = visibleSlides;
        this.CheckEdges();
    }
    SetWidth(width) {
        this._containerWidth = width;
        this._renderer.setStyle(this._element.nativeElement, "width", width.toString() + "px");
    }
    SetSlideWidth(slideWidth) {
        this._slideWidth = slideWidth;
    }
    SetLeft(left) {
        this._renderer.setStyle(this._element.nativeElement, "left", left.toString() + "px");
    }
    onNavigateLeft() {
        let SlideContainerLeft = this._element.nativeElement.offsetLeft;
        if (SlideContainerLeft >= 0) {
            this.NavigateLeft(0);
        }
        else if (SlideContainerLeft <= -(this._slideWidth * this.VisibleSlides)) {
            this.NavigateLeft(SlideContainerLeft + (this._slideWidth * this.VisibleSlides));
        }
        else if (SlideContainerLeft > (-(this._slideWidth * this.VisibleSlides)) && SlideContainerLeft < 0) {
            this.NavigateLeft(0);
        }
    }
    onNavigateRight() {
        let SlideContainerLeft = this._element.nativeElement.offsetLeft;
        if (SlideContainerLeft !== 0 && SlideContainerLeft <= -(this._containerWidth - (this._slideWidth * this.VisibleSlides))) {
            this.NavigateLeft(-(this._containerWidth - (this._slideWidth * this.VisibleSlides)));
        }
        else if (this._containerWidth + SlideContainerLeft >= 2 * (this._slideWidth * this.VisibleSlides)) {
            this.NavigateLeft(SlideContainerLeft + (-this._slideWidth * this.VisibleSlides));
        }
        else if (this._containerWidth + SlideContainerLeft >= (this._slideWidth * this.VisibleSlides)
            && this._containerWidth + SlideContainerLeft < 2 * (this._slideWidth * this.VisibleSlides)) {
            let toleft = ((this._containerWidth - this._slideWidth * this.VisibleSlides) + SlideContainerLeft);
            this.NavigateLeft(SlideContainerLeft + (-toleft));
        }
    }
    NavigateLeft(left) {
        this._containerOffsetLeft = left;
        this.SetLeft(left);
    }
    CheckEdges() {
        let SlideContainerLeft = this._element.nativeElement.offsetLeft;
        if (SlideContainerLeft >= 0) {
            this.NavigateLeft(0);
            return;
        }
        if (SlideContainerLeft < -(this._containerWidth - (this._slideWidth * this.VisibleSlides))) {
            this.NavigateLeft(-(this._containerWidth - (this._slideWidth * this.VisibleSlides)));
            return;
        }
        //Prevent half sides of other slides from showing when window is resized
        if (SlideContainerLeft % (this._slideWidth * this.VisibleSlides) != 0) {
            let remainder = SlideContainerLeft % this._slideWidth;
            if (Math.abs(remainder) >= (this._slideWidth / 2)) {
                this.NavigateLeft(SlideContainerLeft + -(this._slideWidth + remainder));
            }
            else {
                this.NavigateLeft(SlideContainerLeft + (-remainder));
            }
        }
    }
    get IsAtStart() {
        return Math.abs(this._containerOffsetLeft) == 0;
    }
    get IsAtEnd() {
        let isLastSlide = Math.abs(Math.trunc(this._containerOffsetLeft)) == Math.trunc((this._containerWidth - (this._slideWidth * this.VisibleSlides)));
        let smallContainer = this._containerWidth < (this._slideWidth * this.VisibleSlides);
        return isLastSlide || smallContainer;
    }
};
SlidesContainerComponent = __decorate([
    Component({
        selector: 'slides-container',
        template: "<ng-content></ng-content>",
        host: {
            '(swiperight)': "onNavigateLeft()",
            '(swipeleft)': "onNavigateRight()",
        },
        encapsulation: ViewEncapsulation.None,
        styles: ["slides-container{display:block;height:100%;position:relative;left:0;transition:left .4s ease-out}slides-container .slides{display:flex;flex:1;height:100%}"]
    }),
    __metadata("design:paramtypes", [Renderer2, ElementRef])
], SlidesContainerComponent);

let HsNavLeftComponent = class HsNavLeftComponent {
    constructor(_renderer, _element) {
        this._renderer = _renderer;
        this._element = _element;
    }
};
HsNavLeftComponent = __decorate([
    Component({
        selector: 'hs-nav-left',
        template: '<ng-content></ng-content>'
    }),
    __metadata("design:paramtypes", [Renderer2, ElementRef])
], HsNavLeftComponent);
let HsNavRightComponent = class HsNavRightComponent {
    constructor(_renderer, _element) {
        this._renderer = _renderer;
        this._element = _element;
    }
};
HsNavRightComponent = __decorate([
    Component({
        selector: 'hs-nav-right',
        template: '<ng-content></ng-content>'
    }),
    __metadata("design:paramtypes", [Renderer2, ElementRef])
], HsNavRightComponent);
let HorizontalSliderComponent = class HorizontalSliderComponent {
    constructor(_renderer, _element) {
        this._renderer = _renderer;
        this._element = _element;
        this._isContentInit = false;
        this.HideNav = false;
        this.KeyboardNavActive = true;
        this.XsVisibleSlides = 1;
        this.SmVisibleSlides = 2;
        this.MdVisibleSlides = 3;
        this.LgVisibleSlides = 4;
    }
    set keyboardNav(v) {
        if (v) {
            this._keydownSubscription = fromEvent(document, 'keydown')
                .pipe(debounceTime(150))
                .subscribe((e) => {
                if (!this.KeyboardNavActive)
                    return;
                if (e.keyCode == 39) {
                    this.onNavigateRight();
                }
                else if (e.keyCode == 37) {
                    this.onNavigateLeft();
                }
            });
        }
    }
    ngAfterViewInit() {
        if (!this.HideNav) {
            this._renderer.addClass(this._element.nativeElement, "hs-navigation");
        }
        this.CalulateEngine();
    }
    ngAfterContentInit() {
        this.HsSlideComponents.notifyOnChanges();
        this.HsSlideComponents.changes.subscribe((e) => {
            this._isContentInit = true;
            this.CalulateEngine();
        });
    }
    onNavigateLeft() {
        this.SlidesContainer.onNavigateLeft();
    }
    onNavigateRight() {
        this.SlidesContainer.onNavigateRight();
    }
    ngOnInit() {
        this._onResizeListenerHandler = window.addEventListener("resize", () => {
            this.CalulateEngine();
        });
    }
    ngOnDestroy() {
        window.removeEventListener("resize", this._onResizeListenerHandler);
        if (this._keydownSubscription && !this._keydownSubscription.closed) {
            this._keydownSubscription.unsubscribe();
        }
    }
    get IsAtStart() {
        return this.SlidesContainer.IsAtStart;
    }
    get IsAtEnd() {
        return this.SlidesContainer.IsAtEnd;
    }
    CalulateEngine() {
        let WindowWidth = window.innerWidth;
        if (WindowWidth > 0 && WindowWidth <= 600) {
            this.SetSizes(this.XsVisibleSlides);
        }
        else if (WindowWidth > 600 && WindowWidth < 960) {
            this.SetSizes(this.SmVisibleSlides);
        }
        else if (WindowWidth > 960 && WindowWidth < 1280) {
            this.SetSizes(this.MdVisibleSlides);
        }
        else if (WindowWidth > 1280) {
            this.SetSizes(this.LgVisibleSlides);
        }
    }
    SetSizes(visibleSlides) {
        this._slideWidth = this._element.nativeElement.clientWidth / visibleSlides;
        this.SlidesContainer.SetSlideWidth(this._slideWidth);
        this.SlidesContainer.SetWidth(this._slideWidth * this.HsSlideComponents.length);
        this.HsSlideComponents.toArray().forEach(element => {
            element.SetWidth(this._slideWidth);
        });
        this.SlidesContainer.SetVisibleSlides(visibleSlides);
    }
};
__decorate([
    Input("hide-nav"),
    __metadata("design:type", Boolean)
], HorizontalSliderComponent.prototype, "HideNav", void 0);
__decorate([
    Input("keyboard-nav-active"),
    __metadata("design:type", Boolean)
], HorizontalSliderComponent.prototype, "KeyboardNavActive", void 0);
__decorate([
    Input("keyboard-nav"),
    __metadata("design:type", Boolean),
    __metadata("design:paramtypes", [Boolean])
], HorizontalSliderComponent.prototype, "keyboardNav", null);
__decorate([
    ContentChildren(HsSlideComponent),
    __metadata("design:type", QueryList)
], HorizontalSliderComponent.prototype, "HsSlideComponents", void 0);
__decorate([
    ViewChild(SlidesContainerComponent),
    __metadata("design:type", SlidesContainerComponent)
], HorizontalSliderComponent.prototype, "SlidesContainer", void 0);
__decorate([
    Input("xs-visible-slides"),
    __metadata("design:type", Number)
], HorizontalSliderComponent.prototype, "XsVisibleSlides", void 0);
__decorate([
    Input("sm-visible-slides"),
    __metadata("design:type", Number)
], HorizontalSliderComponent.prototype, "SmVisibleSlides", void 0);
__decorate([
    Input("md-visible-slides"),
    __metadata("design:type", Number)
], HorizontalSliderComponent.prototype, "MdVisibleSlides", void 0);
__decorate([
    Input("lg-visible-slides"),
    __metadata("design:type", Number)
], HorizontalSliderComponent.prototype, "LgVisibleSlides", void 0);
HorizontalSliderComponent = __decorate([
    Component({
        selector: 'horizontal-slider, [horizontal-slider]',
        template: "<div class=\"horizontal-slider\">\r\n    <div class=\"horizontal-slider-wrapper\">\r\n        <slides-container>\r\n            <div class=\"slides\">\r\n                <ng-content select=\"hs-slide\"></ng-content>\r\n            </div>\r\n        </slides-container>\r\n    </div>\r\n\r\n    <div class=\"nav nav-left\" (click)=\"onNavigateLeft()\" *ngIf=\"!HideNav && _isContentInit\"\r\n        [class.disabled]=\"IsAtStart\">\r\n        <ng-content select=\"hs-nav-left\"></ng-content>\r\n    </div>\r\n    <div class=\"nav nav-right\" (click)=\"onNavigateRight()\" *ngIf=\"!HideNav && _isContentInit\"\r\n        [class.disabled]=\"IsAtEnd\">\r\n        <ng-content select=\"hs-nav-right\"></ng-content>\r\n    </div>\r\n</div>",
        encapsulation: ViewEncapsulation.None,
        styles: ["[horizontal-slider],horizontal-slider{display:flex;flex-direction:column;height:100%}[horizontal-slider].hs-navigation,horizontal-slider.hs-navigation{margin-left:34px;margin-right:34px}[horizontal-slider] .horizontal-slider,horizontal-slider .horizontal-slider{position:relative;display:flex;flex-direction:column;flex:1;max-height:100%}[horizontal-slider] .horizontal-slider .horizontal-slider-wrapper,horizontal-slider .horizontal-slider .horizontal-slider-wrapper{position:relative;display:flex;flex-direction:column;flex:1;overflow:hidden}[horizontal-slider] .horizontal-slider .nav,horizontal-slider .horizontal-slider .nav{position:absolute;top:50%;-webkit-transform:translateY(-50%);transform:translateY(-50%);font-size:2.3rem;padding:7px;cursor:pointer;transition:left .31s,right .31s}[horizontal-slider] .horizontal-slider .nav:active,horizontal-slider .horizontal-slider .nav:active{color:rgba(0,0,0,.57)}[horizontal-slider] .horizontal-slider .nav.disabled,horizontal-slider .horizontal-slider .nav.disabled{color:rgba(0,0,0,.17)}[horizontal-slider] .horizontal-slider .nav.disabled.nav-left:hover,horizontal-slider .horizontal-slider .nav.disabled.nav-left:hover{left:-30px}[horizontal-slider] .horizontal-slider .nav.disabled.nav-right:hover,horizontal-slider .horizontal-slider .nav.disabled.nav-right:hover{right:-30px}[horizontal-slider] .horizontal-slider .nav.nav-left,horizontal-slider .horizontal-slider .nav.nav-left{left:-30px}[horizontal-slider] .horizontal-slider .nav.nav-left:hover,horizontal-slider .horizontal-slider .nav.nav-left:hover{left:-34px}[horizontal-slider] .horizontal-slider .nav.nav-right,horizontal-slider .horizontal-slider .nav.nav-right{right:-30px}[horizontal-slider] .horizontal-slider .nav.nav-right:hover,horizontal-slider .horizontal-slider .nav.nav-right:hover{right:-34px}"]
    }),
    __metadata("design:paramtypes", [Renderer2, ElementRef])
], HorizontalSliderComponent);

var HorizontalSliderModule_1;
let HorizontalSliderModule = HorizontalSliderModule_1 = class HorizontalSliderModule {
    static forRoot() {
        return { ngModule: HorizontalSliderModule_1, providers: [] };
    }
};
HorizontalSliderModule = HorizontalSliderModule_1 = __decorate([
    NgModule({
        declarations: [
            HorizontalSliderComponent,
            SlidesContainerComponent,
            HsNavLeftComponent,
            HsNavRightComponent,
            HsSlideComponent
        ],
        imports: [
            BrowserModule
        ],
        exports: [
            HorizontalSliderComponent,
            HsSlideComponent,
            HsNavLeftComponent,
            HsNavRightComponent,
        ],
        providers: [],
    })
], HorizontalSliderModule);

/**
 * Generated bundle index. Do not edit.
 */

export { HorizontalSliderModule, HsNavLeftComponent as ɵa, HsNavRightComponent as ɵb, HorizontalSliderComponent as ɵc, HsSlideComponent as ɵd, SlidesContainerComponent as ɵe };
//# sourceMappingURL=angular2-horizontal-slider.js.map
